//
//  CRClassInfo.h
//  CRModel <https://github.com/ibireme/YYModel>
//
//  Created by ibireme on 15/5/9.
//  Copyright (c) 2015 ibireme.
//
//  This source code is licensed under the MIT-style license found in the
//  LICENSE file in the root directory of this source tree.
//

#import <Foundation/Foundation.h>
#import <objc/runtime.h>

NS_ASSUME_NONNULL_BEGIN

/**
 Type encoding's type.
 */
typedef NS_OPTIONS(NSUInteger, CREncodingType) {
    CREncodingTypeMask       = 0xFF, ///< mask of type value
    CREncodingTypeUnknown    = 0, ///< unknown
    CREncodingTypeVoid       = 1, ///< void
    CREncodingTypeBool       = 2, ///< bool
    CREncodingTypeInt8       = 3, ///< char / BOOL
    CREncodingTypeUInt8      = 4, ///< unsigned char
    CREncodingTypeInt16      = 5, ///< short
    CREncodingTypeUInt16     = 6, ///< unsigned short
    CREncodingTypeInt32      = 7, ///< int
    CREncodingTypeUInt32     = 8, ///< unsigned int
    CREncodingTypeInt64      = 9, ///< long long
    CREncodingTypeUInt64     = 10, ///< unsigned long long
    CREncodingTypeFloat      = 11, ///< float
    CREncodingTypeDouble     = 12, ///< double
    CREncodingTypeLongDouble = 13, ///< long double
    CREncodingTypeObject     = 14, ///< id
    CREncodingTypeClass      = 15, ///< Class
    CREncodingTypeSEL        = 16, ///< SEL
    CREncodingTypeBlock      = 17, ///< block
    CREncodingTypePointer    = 18, ///< void*
    CREncodingTypeStruct     = 19, ///< struct
    CREncodingTypeUnion      = 20, ///< union
    CREncodingTypeCString    = 21, ///< char*
    CREncodingTypeCArray     = 22, ///< char[10] (for example)
    
    CREncodingTypeQualifierMask   = 0xFF00,   ///< mask of qualifier
    CREncodingTypeQualifierConst  = 1 << 8,  ///< const
    CREncodingTypeQualifierIn     = 1 << 9,  ///< in
    CREncodingTypeQualifierInout  = 1 << 10, ///< inout
    CREncodingTypeQualifierOut    = 1 << 11, ///< out
    CREncodingTypeQualifierBycopy = 1 << 12, ///< bycopy
    CREncodingTypeQualifierByref  = 1 << 13, ///< byref
    CREncodingTypeQualifierOneway = 1 << 14, ///< oneway
    
    CREncodingTypePropertyMask         = 0xFF0000, ///< mask of property
    CREncodingTypePropertyReadonly     = 1 << 16, ///< readonly
    CREncodingTypePropertyCopy         = 1 << 17, ///< copy
    CREncodingTypePropertyRetain       = 1 << 18, ///< retain
    CREncodingTypePropertyNonatomic    = 1 << 19, ///< nonatomic
    CREncodingTypePropertyWeak         = 1 << 20, ///< weak
    CREncodingTypePropertyCustomGetter = 1 << 21, ///< getter=
    CREncodingTypePropertyCustomSetter = 1 << 22, ///< setter=
    CREncodingTypePropertyDynamic      = 1 << 23, ///< @dynamic
};

/**
 Get the type from a Type-Encoding string.
 
 @discussion See also:
 https://developer.apple.com/library/mac/documentation/Cocoa/Conceptual/ObjCRuntimeGuide/Articles/ocrtTypeEncodings.html
 https://developer.apple.com/library/mac/documentation/Cocoa/Conceptual/ObjCRuntimeGuide/Articles/ocrtPropertyIntrospection.html
 
 @param typeEncoding  A Type-Encoding string.
 @return The encoding type.
 */
CREncodingType CREncodingGetType(const char *typeEncoding);


/**
 Instance variable information.
 */
@interface CRClassIvarInfo : NSObject
@property (nonatomic, assign, readonly) Ivar ivar;              ///< ivar opaque struct
@property (nonatomic, strong, readonly) NSString *name;         ///< Ivar's name
@property (nonatomic, assign, readonly) ptrdiff_t offset;       ///< Ivar's offset
@property (nonatomic, strong, readonly) NSString *typeEncoding; ///< Ivar's type encoding
@property (nonatomic, assign, readonly) CREncodingType type;    ///< Ivar's type

/**
 Creates and returns an ivar info object.
 
 @param ivar ivar opaque struct
 @return A new object, or nil if an error occurs.
 */
- (instancetype)initWithIvar:(Ivar)ivar;
@end


/**
 Method information.
 */
@interface CRClassMethodInfo : NSObject
@property (nonatomic, assign, readonly) Method method;                  ///< method opaque struct
@property (nonatomic, strong, readonly) NSString *name;                 ///< method name
@property (nonatomic, assign, readonly) SEL sel;                        ///< method's selector
@property (nonatomic, assign, readonly) IMP imp;                        ///< method's implementation
@property (nonatomic, strong, readonly) NSString *typeEncoding;         ///< method's parameter and return types
@property (nonatomic, strong, readonly) NSString *returnTypeEncoding;   ///< return value's type
@property (nullable, nonatomic, strong, readonly) NSArray<NSString *> *argumentTypeEncodings; ///< array of arguments' type

/**
 Creates and returns a method info object.
 
 @param method method opaque struct
 @return A new object, or nil if an error occurs.
 */
- (instancetype)initWithMethod:(Method)method;
@end


/**
 Property information.
 */
@interface CRClassPropertyInfo : NSObject
@property (nonatomic, assign, readonly) objc_property_t property; ///< property's opaque struct
@property (nonatomic, strong, readonly) NSString *name;           ///< property's name
@property (nonatomic, assign, readonly) CREncodingType type;      ///< property's type
@property (nonatomic, strong, readonly) NSString *typeEncoding;   ///< property's encoding value
@property (nonatomic, strong, readonly) NSString *ivarName;       ///< property's ivar name
@property (nullable, nonatomic, assign, readonly) Class cls;      ///< may be nil
@property (nullable, nonatomic, strong, readonly) NSArray<NSString *> *protocols; ///< may nil
@property (nonatomic, assign, readonly) SEL getter;               ///< getter (nonnull)
@property (nonatomic, assign, readonly) SEL setter;               ///< setter (nonnull)

/**
 Creates and returns a property info object.
 
 @param property property opaque struct
 @return A new object, or nil if an error occurs.
 */
- (instancetype)initWithProperty:(objc_property_t)property;
@end


/**
 Class information for a class.
 */
@interface CRClassInfo : NSObject
@property (nonatomic, assign, readonly) Class cls; ///< class object
@property (nullable, nonatomic, assign, readonly) Class superCls; ///< super class object
@property (nullable, nonatomic, assign, readonly) Class metaCls;  ///< class's meta class object
@property (nonatomic, readonly) BOOL isMeta; ///< whether this class is meta class
@property (nonatomic, strong, readonly) NSString *name; ///< class name
@property (nullable, nonatomic, strong, readonly) CRClassInfo *superClassInfo; ///< super class's class info
@property (nullable, nonatomic, strong, readonly) NSDictionary<NSString *, CRClassIvarInfo *> *ivarInfos; ///< ivars
@property (nullable, nonatomic, strong, readonly) NSDictionary<NSString *, CRClassMethodInfo *> *methodInfos; ///< methods
@property (nullable, nonatomic, strong, readonly) NSDictionary<NSString *, CRClassPropertyInfo *> *propertyInfos; ///< properties

/**
 If the class is changed (for example: you add a method to this class with
 'class_addMethod()'), you should call this method to refresh the class info cache.
 
 After called this method, `needUpdate` will returns `YES`, and you should call 
 'classInfoWithClass' or 'classInfoWithClassName' to get the updated class info.
 */
- (void)setNeedUpdate;

/**
 If this method returns `YES`, you should stop using this instance and call
 `classInfoWithClass` or `classInfoWithClassName` to get the updated class info.
 
 @return Whether this class info need update.
 */
- (BOOL)needUpdate;

/**
 Get the class info of a specified Class.
 
 @discussion This method will cache the class info and super-class info
 at the first access to the Class. This method is thread-safe.
 
 @param cls A class.
 @return A class info, or nil if an error occurs.
 */
+ (nullable instancetype)classInfoWithClass:(Class)cls;

/**
 Get the class info of a specified Class.
 
 @discussion This method will cache the class info and super-class info
 at the first access to the Class. This method is thread-safe.
 
 @param className A class name.
 @return A class info, or nil if an error occurs.
 */
+ (nullable instancetype)classInfoWithClassName:(NSString *)className;

@end

NS_ASSUME_NONNULL_END
